//
//  GHPImageView.swift
//  Granja
//
//  Created by Santiago Pavón on 21/10/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import UIKit

enum GHPState {
    case hen
    case egg
    case chicken
    case fried
    case roast
}

class GHPImageView: UIImageView {

    
    // Estado que indica la imagen a mostrar
    var ghpState: GHPState = .hen {
        didSet {
            updateGHPStateImage()
        }
    }
    
    // Inicializador
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        updateGHPStateImage()
        
        // Necesito enterarme de los eventos del usuario
        isUserInteractionEnabled = true
        
        // Crear reconocedor de Tap para ir de gallina a huevo, a pollo, y a gallina
        let tapRec = UITapGestureRecognizer(target: self, action: #selector(processTap(_:)))
        tapRec.numberOfTapsRequired = 1
        addGestureRecognizer(tapRec)
        
        // Crear reconocedor de Long Press para freir huevo
        let lpRec = UILongPressGestureRecognizer(target: self, action: #selector(processLongPress(_:)))
        addGestureRecognizer(lpRec)
        
        // Crear reconocedor de Pan para mover imagen view
        let panRec = UIPanGestureRecognizer(target: self, action: #selector(processPan(_:)))
        addGestureRecognizer(panRec)
        
        // Crear reconocedor de Pinch para escalar imagen view
        let pinchRec = UIPinchGestureRecognizer(target: self, action: #selector(processPinch(_:)))
        addGestureRecognizer(pinchRec)
        
        // Crear reconocedor de Rotation para girar imagen view
        let rotationRec = UIRotationGestureRecognizer(target: self, action: #selector(processRotation(_:)))
        addGestureRecognizer(rotationRec)
    }
    
    
    required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    // En funcion del estado se usa una imagen distinta
    fileprivate func updateGHPStateImage() {
        
        switch ghpState {
        case .hen:
            image = UIImage(named: "gallina")
        case .egg:
            image = UIImage(named: "huevo")
        case .chicken:
            image = UIImage(named: "pollo")
        case .fried:
            image = UIImage(named: "frito")
        case .roast:
            image = UIImage(named: "asado")
        }
        
        if image != nil {
            bounds.size = image!.size
        }
    }
    
    func processTap(_ sender: UITapGestureRecognizer) {
        
        switch ghpState {
        case .hen:
            ghpState = .egg
        case .egg:
            ghpState = .chicken
        case .chicken:
            ghpState = .hen
        case .fried:
            break
        case .roast:
            break
        }
    }
    
    func processLongPress(_ sender: UILongPressGestureRecognizer) {
        
        if sender.state != .began { return }
        
        UIView.transition(with: self,
            duration: 0.5,
            options: .transitionFlipFromLeft,
            animations: {
                switch self.ghpState {
                case .hen:
                    self.ghpState = .roast
                case .egg:
                    self.ghpState = .fried
                case .chicken:
                    break
                case .fried:
                    break
                case .roast:
                    break
                }
            },
            completion: nil)
    }
    
    func processPinch(_ sender: UIPinchGestureRecognizer) {
        let factor = sender.scale
        
        transform = transform.scaledBy(x: factor, y: factor)
        
        sender.scale = 1
    }
    
    
    func processRotation(_ sender: UIRotationGestureRecognizer) {
        let angle = sender.rotation
        
        transform = transform.rotated(by: angle)
        
        sender.rotation = 0
    }
    
    
    func processPan(_ sender: UIPanGestureRecognizer) {
        let pos = sender.translation(in: sender.view!)
        
        transform = transform.translatedBy(x: pos.x, y: pos.y)
        
        sender.setTranslation(CGPoint.zero, in: sender.view)
    }
    
}
